@@ -39,14 +39,15 @@
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/AddonManager.jsm");
 
 [
   ["AllPagesList", "popup_autocomplete", "cmd_openLocation"],
   ["HistoryList", "history-items", "cmd_history"],
   ["BookmarkList", "bookmarks-items", "cmd_bookmarks"],
-//@line 48 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 49 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
   ["RemoteTabsList", "remotetabs-items", "cmd_remoteTabs"]
-//@line 50 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 51 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
 ].forEach(function(aPanel) {
   let [name, id, command] = aPanel;
   XPCOMUtils.defineLazyGetter(window, name, function() {
@@ -63,6 +64,7 @@ let Elements = {};
   ["contentShowing",     "bcast_contentShowing"],
   ["urlbarState",        "bcast_urlbarState"],
   ["stack",              "stack"],
+  ["tabList",            "tabs"],
   ["tabs",               "tabs-container"],
   ["controls",           "browser-controls"],
   ["panelUI",            "panel-container"],
@@ -114,18 +116,22 @@ var BrowserUI = {
   },
 
   _titleChanged: function(aBrowser) {
+    let url = this.getDisplayURI(aBrowser);
+    let caption = aBrowser.contentTitle || url;
+
+    if (aBrowser.contentTitle == "" && !Util.isURLEmpty(aBrowser.userTypedValue))
+      caption = aBrowser.userTypedValue;
+    else if (Util.isURLEmpty(url))
+      caption = "";
+
+    let tab = Browser.getTabForBrowser(aBrowser);
+    if (tab)
+      tab.chromeTab.updateTitle(caption);
+
     let browser = Browser.selectedBrowser;
     if (browser && aBrowser != browser)
       return;
 
-    let url = this.getDisplayURI(browser);
-    let caption = browser.contentTitle || url;
-
-    if (browser.contentTitle == "" && !Util.isURLEmpty(browser.userTypedValue))
-      caption = browser.userTypedValue;
-    else if (Util.isURLEmpty(url))
-      caption = "";
-
     if (caption) {
       this._title.value = caption;
       this._title.classList.remove("placeholder");
@@ -160,7 +166,7 @@ var BrowserUI = {
 
   _updateToolbar: function _updateToolbar() {
     let mode = Elements.urlbarState.getAttribute("mode");
-    if (mode == "edit" && this.activePanel)
+    if (mode == "edit" && AwesomeScreen.activePanel)
       return;
 
     if (Browser.selectedTab.isLoading() && mode != "loading")
@@ -185,6 +191,8 @@ var BrowserUI = {
   },
 
   lockToolbar: function lockToolbar() {
+    if (Util.isTablet())
+      return;
     this._toolbarLocked++;
     document.getElementById("toolbar-moveable-container").top = "0";
     if (this._toolbarLocked == 1)
@@ -201,7 +209,7 @@ var BrowserUI = {
   },
 
   _setURL: function _setURL(aURL) {
-    if (this.activePanel)
+    if (AwesomeScreen.activePanel)
       this._edit.defaultValue = aURL;
     else
       this._edit.value = aURL;
@@ -227,12 +235,12 @@ var BrowserUI = {
     // During an awesome search we always show the popup_autocomplete/AllPagesList
     // panel since this looks in every places and the rationale behind typing
     // is to find something, whereever it is.
-    if (this.activePanel != AllPagesList) {
+    if (AwesomeScreen.activePanel != AllPagesList) {
       let inputField = this._edit;
       let oldClickSelectsAll = inputField.clickSelectsAll;
       inputField.clickSelectsAll = false;
 
-      this.activePanel = AllPagesList;
+      AwesomeScreen.activePanel = AllPagesList;
 
       // changing the searchString property call updateAwesomeHeader again
       inputField.controller.searchString = aString;
@@ -248,8 +256,8 @@ var BrowserUI = {
 
   _closeOrQuit: function _closeOrQuit() {
     // Close active dialog, if we have one. If not then close the application.
-    if (this.activePanel) {
-      this.activePanel = null;
+    if (AwesomeScreen.activePanel) {
+      AwesomeScreen.activePanel = null;
     } else if (this.activeDialog) {
       this.activeDialog.close();
     } else {
@@ -259,58 +267,6 @@ var BrowserUI = {
     }
   },
 
-  _activePanel: null,
-  get activePanel() {
-    return this._activePanel;
-  },
-
-  set activePanel(aPanel) {
-    if (this._activePanel == aPanel)
-      return;
-
-    let awesomePanel = document.getElementById("awesome-panels");
-    let awesomeHeader = document.getElementById("awesome-header");
-
-    let willShowPanel = (!this._activePanel && aPanel);
-    if (willShowPanel) {
-      this.pushDialog(aPanel);
-      this._edit.attachController();
-      this._editURI();
-      awesomePanel.hidden = awesomeHeader.hidden = false;
-    };
-
-    if (aPanel) {
-      aPanel.open();
-      if (this._edit.value == "")
-        this._showURI();
-    }
-
-    let willHidePanel = (this._activePanel && !aPanel);
-    if (willHidePanel) {
-      awesomePanel.hidden = true;
-      awesomeHeader.hidden = false;
-      this._edit.reset();
-      this._edit.detachController();
-      this.popDialog();
-    }
-
-    if (this._activePanel)
-      this._activePanel.close();
-
-    // The readOnly state of the field enabled/disabled the VKB
-    let isReadOnly = !(aPanel == AllPagesList && Util.isPortrait() && (willShowPanel || !this._edit.readOnly));
-    this._edit.readOnly = isReadOnly;
-    if (isReadOnly)
-      this._edit.blur();
-
-    this._activePanel = aPanel;
-    if (willHidePanel || willShowPanel) {
-      let event = document.createEvent("UIEvents");
-      event.initUIEvent("NavigationPanel" + (willHidePanel ? "Hidden" : "Shown"), true, true, window, false);
-      window.dispatchEvent(event);
-    }
-  },
-
   get activeDialog() {
     // Return the topmost dialog
     if (this._dialogs.length)
@@ -355,6 +311,12 @@ var BrowserUI = {
     this._dispatchPopupChanged(false);
   },
 
+  // Will the on-screen keyboard cover the whole screen when opened?
+  _isKeyboardFullscreen: function _isKeyboardFullscreen() {
+//@line 329 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
+    return false;
+  },
+
   _dispatchPopupChanged: function _dispatchPopupChanged(aVisible) {
     let stack = document.getElementById("stack");
     let event = document.createEvent("UIEvents");
@@ -412,25 +374,10 @@ var BrowserUI = {
     return this._toolbarH;
   },
 
-  get sidebarW() {
-    delete this._sidebarW;
-    return this._sidebarW = Elements.controls.getBoundingClientRect().width;
-  },
-
-  get starButton() {
-    delete this.starButton;
-    return this.starButton = document.getElementById("tool-star");
-  },
-
   sizeControls: function(windowW, windowH) {
     // tabs
-    document.getElementById("tabs").resize();
-
-    // awesomebar and related panels
-    let popup = document.getElementById("awesome-panels");
-    popup.top = this.toolbarH;
-    popup.height = windowH - this.toolbarH;
-    popup.width = windowW;
+    Elements.tabList.resize();
+    AwesomeScreen.doResize(windowW, windowH);
 
     // content navigator helper
     document.getElementById("content-navigator").contentHasChanged();
@@ -450,7 +397,7 @@ var BrowserUI = {
     window.addEventListener("NavigationPanelHidden", this, false);
 
     Elements.browsers.addEventListener("PanFinished", this, true);
-//@line 456 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 415 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
 
     // listen content messages
     messageManager.addMessageListener("DOMLinkAdded", this);
@@ -470,6 +417,9 @@ var BrowserUI = {
     // listening AppCommand to handle special keys
     window.addEventListener("AppCommand", this, true);
 
+    // Initialize the number of tabs in toolbar
+    TabsPopup.init();
+
     // We can delay some initialization until after startup.  We wait until
     // the first page is shown, then dispatch a UIReadyDelayed event.
     messageManager.addMessageListener("pageshow", function() {
@@ -485,6 +435,11 @@ var BrowserUI = {
       }, 0);
     });
 
+    // Only load IndexedDB.js when we actually need it. A general fix will happen in bug 647079.
+    messageManager.addMessageListener("IndexedDB:Prompt", function(aMessage) {
+      return IndexedDB.receiveMessage(aMessage);
+    });
+
     // Delay the panel UI and Sync initialization.
     window.addEventListener("UIReadyDelayed", function(aEvent) {
       window.removeEventListener(aEvent.type, arguments.callee, false);
@@ -506,15 +461,12 @@ var BrowserUI = {
       DownloadsView.init();
       ConsoleView.init();
 
-      // Pre-start the content process
-      Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime)
-          .ensureContentProcess();
-
-//@line 516 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 479 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
       // Init the sync system
       WeaveGlue.init();
-//@line 519 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 482 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
 
+      Services.prefs.addObserver("browser.ui.layout.tablet", BrowserUI, false);
       Services.obs.addObserver(BrowserSearch, "browser-search-engine-modified", false);
       messageManager.addMessageListener("Browser:MozApplicationManifest", OfflineApps);
 
@@ -522,26 +474,23 @@ var BrowserUI = {
       BadgeHandlers.register(BrowserUI._edit.popup);
       FormHelperUI.init();
       FindHelperUI.init();
-      PageActions.init();
       FullScreenVideo.init();
       NewTabPopup.init();
-      CharsetMenu.init();
+      WebappsUI.init();
+      CapturePickerUI.init();
 
       // If some add-ons were disabled during during an application update, alert user
-      if (Services.prefs.prefHasUserValue("extensions.disabledAddons")) {
-        let addons = Services.prefs.getCharPref("extensions.disabledAddons").split(",");
-        if (addons.length > 0) {
-          let disabledStrings = Strings.browser.GetStringFromName("alertAddonsDisabled");
-          let label = PluralForm.get(addons.length, disabledStrings).replace("#1", addons.length);
-          let image = "chrome://browser/skin/images/alert-addons-30.png";
+      let addonIDs = AddonManager.getStartupChanges("disabled");
+      if (addonIDs.length > 0) {
+        let disabledStrings = Strings.browser.GetStringFromName("alertAddonsDisabled");
+        let label = PluralForm.get(addonIDs.length, disabledStrings).replace("#1", addonIDs.length);
+        let image = "chrome://browser/skin/images/alert-addons-30.png";
 
-          let alerts = Cc["@mozilla.org/toaster-alerts-service;1"].getService(Ci.nsIAlertsService);
-          alerts.showAlertNotification(image, Strings.browser.GetStringFromName("alertAddons"), label, false, "", null);
-        }
-        Services.prefs.clearUserPref("extensions.disabledAddons");
+        let alerts = Cc["@mozilla.org/toaster-alerts-service;1"].getService(Ci.nsIAlertsService);
+        alerts.showAlertNotification(image, Strings.browser.GetStringFromName("alertAddons"), label, false, "", null);
       }
 
-//@line 551 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 512 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
     }, false);
 
     let panels = document.getElementById("panel-items");
@@ -559,16 +508,51 @@ var BrowserUI = {
         window[viewName].delayedInit();
     }, true);
 
-//@line 577 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 538 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
   },
 
   uninit: function() {
     Services.obs.removeObserver(BrowserSearch, "browser-search-engine-modified");
+    Services.prefs.removeObserver("browser.ui.layout.tablet", BrowserUI);
     messageManager.removeMessageListener("Browser:MozApplicationManifest", OfflineApps);
     ExtensionsView.uninit();
     ConsoleView.uninit();
   },
 
+  observe: function observe(aSubject, aTopic, aData) {
+    if (aTopic == "nsPref:changed" && aData == "browser.ui.layout.tablet")
+      this.updateTabletLayout();
+  },
+
+  updateTabletLayout: function updateTabletLayout() {
+    let wasTablet = Elements.urlbarState.hasAttribute("tablet");
+    let isTablet = Util.isTablet({ forceUpdate: true });
+    if (wasTablet == isTablet)
+      return;
+
+    if (isTablet) {
+      this.unlockToolbar();
+      Elements.urlbarState.setAttribute("tablet", "true");
+    } else {
+      Elements.urlbarState.removeAttribute("tablet");
+    }
+
+    // Tablet mode changes the size of the thumbnails
+    // in the tabs container. Hence we have to force a
+    // thumbnail update on all tabs.
+    setTimeout(function(self) {
+      self._updateAllTabThumbnails();
+    }, 0, this);
+  },
+
+  _updateAllTabThumbnails: function() {
+    let tabs = Browser.tabs;
+
+    tabs.forEach(function(tab) {
+      tab.updateThumbnail({ force: true });
+    });
+  },
+
   update: function(aState) {
     let browser = Browser.selectedBrowser;
 
@@ -642,16 +626,23 @@ var BrowserUI = {
     // Make sure we're online before attempting to load
     Util.forceOnline();
 
-    // Give the new page lots of room
-    Browser.hideSidebars();
+    // Close the autocomplete panel and quickly update the urlbar value
     this.closeAutoComplete();
-
     this._edit.value = aURI;
 
     let postData = {};
     aURI = Browser.getShortcutOrURI(aURI, postData);
     Browser.loadURI(aURI, { flags: Ci.nsIWebNavigation.LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP, postData: postData });
 
+    // If a page goes from remote (about:blank) to local (about:home), fennec
+    // create a tab for the local page and then close the about:blank tab.
+    // If the newly created tab spawn a new column the sidebars position can
+    // be messed up. Hopefully, the viewable area should be maximized when a
+    // new page is opened, so a call to Browser.hideSidebars() fill this
+    // requirement and fix the sidebars position.
+    Browser.hideSidebars();
+    Elements.tabList.removeClosedTab();
+
     // Delay doing the fixup so the raw URI is passed to loadURIWithFlags
     // and the proper third-party fixup can be done
     let fixupFlags = Ci.nsIURIFixup.FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP;
@@ -667,18 +658,20 @@ var BrowserUI = {
 
     this.hidePanel();
     this._hidePopup();
-    this.activePanel = AllPagesList;
+    if (this.activeDialog)
+      this.activeDialog.close();
+    AwesomeScreen.activePanel = AllPagesList;
   },
 
   closeAutoComplete: function closeAutoComplete() {
     if (this.isAutoCompleteOpen())
       this._edit.popup.closePopup();
 
-    this.activePanel = null;
+    AwesomeScreen.activePanel = null;
   },
 
   isAutoCompleteOpen: function isAutoCompleteOpen() {
-    return this.activePanel == AllPagesList;
+    return AwesomeScreen.activePanel == AllPagesList;
   },
 
   doOpenSearch: function doOpenSearch(aName) {
@@ -694,9 +687,10 @@ var BrowserUI = {
 
     let engine = Services.search.getEngineByName(aName);
     let submission = engine.getSubmission(searchValue, null);
-    Browser.selectedBrowser.userTypedValue = submission.uri.spec;
     Browser.loadURI(submission.uri.spec, { postData: submission.postData });
 
+    // loadURI may open a new tab, so get the selectedBrowser afterward.
+    Browser.selectedBrowser.userTypedValue = submission.uri.spec;
     this._titleChanged(Browser.selectedBrowser);
   },
 
@@ -708,18 +702,25 @@ var BrowserUI = {
   updateStar: function() {
     let uri = getBrowser().currentURI;
     if (uri.spec == "about:blank") {
-      this.starButton.removeAttribute("starred");
+      this._setStar(false);
       return;
     }
 
-    PlacesUtils.asyncGetBookmarkIds(uri, function (aItemIds) {
-      if (aItemIds.length)
-        this.starButton.setAttribute("starred", "true");
-      else
-        this.starButton.removeAttribute("starred");
+    PlacesUtils.asyncGetBookmarkIds(uri, function(aItemIds) {
+      this._setStar(aItemIds.length > 0)
     }, this);
   },
 
+  _setStar: function _setStar(aIsStarred) {
+    let buttons = document.getElementsByClassName("tool-star");
+    for (let i = 0; i < buttons.length; i++) {
+      if (aIsStarred)
+        buttons[i].setAttribute("starred", "true");
+      else
+        buttons[i].removeAttribute("starred");
+    }
+  },
+
   newTab: function newTab(aURI, aOwner) {
     aURI = aURI || "about:blank";
     let tab = Browser.addTab(aURI, true, aOwner);
@@ -756,8 +757,9 @@ var BrowserUI = {
   },
 
   selectTab: function selectTab(aTab) {
-    this.activePanel = null;
+    AwesomeScreen.activePanel = null;
     Browser.selectedTab = aTab;
+    Elements.tabList.removeClosedTab();
   },
 
   undoCloseTab: function undoCloseTab(aIndex) {
@@ -777,8 +779,8 @@ var BrowserUI = {
   },
 
   showPanel: function showPanel(aPanelId) {
-    if (this.activePanel)
-      this.activePanel = null; // Hide the awesomescreen.
+    if (AwesomeScreen.activePanel)
+      AwesomeScreen.activePanel = null; // Hide the awesomescreen.
 
     Elements.panelUI.left = 0;
     Elements.panelUI.hidden = false;
@@ -812,8 +814,8 @@ var BrowserUI = {
 
   switchTask: function switchTask() {
     try {
-      let phone = Cc["@mozilla.org/phone/support;1"].createInstance(Ci.nsIPhoneSupport);
-      phone.switchTask();
+      let shell = Cc["@mozilla.org/browser/shell-service;1"].createInstance(Ci.nsIShellService);
+      shell.switchTask();
     } catch(e) { }
   },
 
@@ -826,16 +828,16 @@ var BrowserUI = {
       return;
     }
 
-    // Check active panel
-    if (this.activePanel) {
-      this.activePanel = null;
+    // Check open dialogs
+    let dialog = this.activeDialog;
+    if (dialog && dialog != AwesomeScreen.activePanel) {
+      dialog.close();
       return;
     }
 
-    // Check open dialogs
-    let dialog = this.activeDialog;
-    if (dialog) {
-      dialog.close();
+    // Check active panel
+    if (AwesomeScreen.activePanel) {
+      AwesomeScreen.activePanel = null;
       return;
     }
 
@@ -864,9 +866,11 @@ var BrowserUI = {
     if (browser.canGoBack) {
       browser.goBack();
     } else if (tab.owner) {
+      // When going back, always return to the owner (not a sibling).
+      Browser.selectedTab = tab.owner;
       this.closeTab(tab);
     }
-//@line 890 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 906 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
   },
 
   handleEvent: function handleEvent(aEvent) {
@@ -876,30 +880,32 @@ var BrowserUI = {
         this._tabSelect(aEvent);
         break;
       case "TabOpen":
-      case "TabRemove":
-      {
-        // Workaround to hide the tabstrip if it is partially visible
-        // See bug 524469 and bug 626660
-        let [tabsVisibility,,,] = Browser.computeSidebarVisibility();
-        if (tabsVisibility > 0.0 && tabsVisibility < 1.0)
-          Browser.hideSidebars();
-
+        Elements.tabList.removeClosedTab();
+        Browser.hidePartialTabSidebar();
         break;
-      }
-      case "PanFinished":
-        let tabs = document.getElementById("tabs");
+      case "TabRemove":
+        Browser.hidePartialTabSidebar();
+        break;
+      case "PanFinished": {
+        let tabs = Elements.tabList;
         let [tabsVisibility,,oldLeftWidth, oldRightWidth] = Browser.computeSidebarVisibility();
         if (tabsVisibility == 0.0 && tabs.hasClosedTab) {
           let { x: x1, y: y1 } = Browser.getScrollboxPosition(Browser.controlsScrollboxScroller);
           tabs.removeClosedTab();
 
-          let [,, leftWidth, rightWidth] = Browser.computeSidebarVisibility();
-          let delta = (oldLeftWidth - leftWidth) || (oldRightWidth - rightWidth);
-          x1 += (x1 == leftWidth) ? delta : -delta;
+          // If the tabs sidebar lives on the left side of the window, width
+          // variation should be taken into account to reposition the sidebars
+          if (tabs.getBoundingClientRect().left < 0) {
+            let [,, leftWidth, rightWidth] = Browser.computeSidebarVisibility();
+            let delta = (oldLeftWidth - leftWidth) || (oldRightWidth - rightWidth);
+            x1 += (x1 == leftWidth) ? delta : -delta;
+          }
+
           Browser.controlsScrollboxScroller.scrollTo(x1, 0);
           Browser.tryFloatToolbar(0, 0);
         }
         break;
+      }
       case "SizeChanged":
         this.sizeControls(ViewableAreaObserver.width, ViewableAreaObserver.height);
         break;
@@ -915,7 +921,7 @@ var BrowserUI = {
             this.doCommand("cmd_menu");
             break;
           case "Search":
-            if (!this.activePanel)
+            if (!AwesomeScreen.activePanel)
               AllPagesList.doCommand();
             else
               this.doCommand("cmd_opensearch");
@@ -980,6 +986,35 @@ var BrowserUI = {
         return this._domWindowClose(browser);
         break;
       case "DOMLinkAdded":
+        // checks for an icon to use for a web app
+        // apple-touch-icon size is 57px and default size is 16px
+        let rel = json.rel.toLowerCase().split(" ");
+        if (rel.indexOf("icon") != -1) {
+          // We use the sizes attribute if available
+          // see http://www.whatwg.org/specs/web-apps/current-work/multipage/links.html#rel-icon
+          let size = 16;
+          if (json.sizes) {
+            let sizes = json.sizes.toLowerCase().split(" ");
+            sizes.forEach(function(item) {
+              if (item != "any") {
+                let [w, h] = item.split("x");
+                size = Math.max(Math.min(w, h), size);
+              }
+            });
+          }
+          if (size > browser.appIcon.size) {
+            browser.appIcon.href = json.href;
+            browser.appIcon.size = size;
+          }
+        }
+        else if ((rel.indexOf("apple-touch-icon") != -1) && (browser.appIcon.size < 57)) {
+          // XXX should we support apple-touch-icon-precomposed ?
+          // see http://developer.apple.com/safari/library/documentation/appleapplications/reference/safariwebcontent/configuringwebapplications/configuringwebapplications.html
+          browser.appIcon.href = json.href;
+          browser.appIcon.size = 57;
+        }
+
+        // Handle favicon changes
         if (Browser.selectedBrowser == browser)
           this._updateIcon(Browser.selectedBrowser.mIconURL);
         break;
@@ -997,7 +1032,7 @@ var BrowserUI = {
         stmt.finalize();
 
         let download = dm.getDownload(json.id);
-//@line 1035 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/browser-ui.js"
+//@line 1082 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/chrome/content/browser-ui.js"
         try {
           DownloadsView.downloadCompleted(download);
           let element = DownloadsView.getElementForDownload(json.id);
@@ -1038,6 +1073,7 @@ var BrowserUI = {
       case "cmd_quit":
       case "cmd_close":
       case "cmd_menu":
+      case "cmd_showTabs":
       case "cmd_newTab":
       case "cmd_closeTab":
       case "cmd_undoCloseTab":
@@ -1059,8 +1095,13 @@ var BrowserUI = {
   },
 
   isCommandEnabled : function(cmd) {
+    // disable all commands during the first-run sidebar discovery
+    let broadcaster = document.getElementById("bcast_uidiscovery");
+    if (broadcaster && broadcaster.getAttribute("mode") == "discovery")
+      return false;
+
     let elem = document.getElementById(cmd);
-    if (elem && (elem.getAttribute("disabled") == "true"))
+    if (elem && elem.getAttribute("disabled") == "true")
       return false;
     return true;
   },
@@ -1101,8 +1142,7 @@ var BrowserUI = {
       case "cmd_star":
       {
         BookmarkPopup.toggle();
-        if (!this.starButton.hasAttribute("starred"))
-          this.starButton.setAttribute("starred", "true");
+        this._setStar(true);
 
         let bookmarkURI = browser.currentURI;
         PlacesUtils.asyncGetBookmarkIds(bookmarkURI, function (aItemIds) {
@@ -1131,39 +1171,25 @@ var BrowserUI = {
         BrowserSearch.toggle();
         break;
       case "cmd_bookmarks":
-        this.activePanel = BookmarkList;
+        AwesomeScreen.activePanel = BookmarkList;
         break;
       case "cmd_history":
-        this.activePanel = HistoryList;
+        AwesomeScreen.activePanel = HistoryList;
         break;
       case "cmd_remoteTabs":
-        // remove the checked state set by the click it will be reset by setting
-        // checked on the command element if we decide to show this panel (see AwesomePanel.js)
-        document.getElementById("remotetabs-button").removeAttribute("checked");
-
         if (Weave.Status.checkSetup() == Weave.CLIENT_NOT_CONFIGURED) {
-          this.activePanel = null;
-
+          // We have to set activePanel before showing sync's dialog
+          // to make the sure the dialog stacking is correct.
+          AwesomeScreen.activePanel = RemoteTabsList;
           WeaveGlue.open();
-        } else if (!Weave.Service.isLoggedIn) {
-          this.activePanel = null;
-
-          BrowserUI.showPanel("prefs-container");
-          let prefsBox = document.getElementById("prefs-list");
-          let syncArea = document.getElementById("prefs-sync");
-          if (prefsBox && syncArea) {
-            let prefsBoxY = prefsBox.firstChild.boxObject.screenY;
-            let syncAreaY = syncArea.boxObject.screenY;
-            setTimeout(function() {
-              prefsBox.scrollBoxObject.scrollTo(0, syncAreaY - prefsBoxY);
-            }, 0);
-          }
         } else {
-          this.activePanel = RemoteTabsList;
+          AwesomeScreen.activePanel = RemoteTabsList;
         }
+
         break;
       case "cmd_quit":
-        GlobalOverlay.goQuitApplication();
+        // Only close one window
+        this._closeOrQuit();
         break;
       case "cmd_close":
         this._closeOrQuit();
@@ -1171,6 +1197,9 @@ var BrowserUI = {
       case "cmd_menu":
         AppMenu.toggle();
         break;
+      case "cmd_showTabs":
+        TabsPopup.toggle();
+        break;
       case "cmd_newTab":
         this.newTab();
         break;
