@@ -37,9 +37,9 @@
 
 Components.utils.import("resource://gre/modules/DownloadUtils.jsm");
 
-//@line 43 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/downloads.js"
+//@line 43 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/downloads.js"
 const URI_GENERIC_ICON_DOWNLOAD = "chrome://browser/skin/images/alert-downloads-30.png";
-//@line 45 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/downloads.js"
+//@line 45 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/downloads.js"
 
 var DownloadsView = {
   _initialized: false,
@@ -87,6 +87,7 @@ var DownloadsView = {
       item.setAttribute("downloadID", aAttrs.id);
       item.setAttribute("iconURL", "moz-icon://" + aAttrs.file + "?size=32");
       item.setAttribute("lastSeconds", Infinity);
+      item.setAttribute("class", "panel-listitem");
   
       // Initialize more complex attributes
       this._updateTime(item);
@@ -348,7 +349,7 @@ var DownloadsView = {
     }
     else if (today - end < (24 * 60 * 60 * 1000)) {
       // Download finished after yesterday started, show yesterday
-      dateTime = strings.GetStringFromName("donwloadsYesterday");
+      dateTime = strings.GetStringFromName("downloadsYesterday");
     }
     else if (today - end < (6 * 24 * 60 * 60 * 1000)) {
       // Download finished after last week started, show day of week
@@ -599,11 +600,11 @@ AlertDownloadProgressListener.prototype = {
       Cc["@mozilla.org/download-manager;1"].getService(Ci.nsIDownloadManager).cancelDownload(aDownload.id);
     }
 
-//@line 614 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/downloads.js"
+//@line 615 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/downloads.js"
   },
 
   onDownloadStateChange: function(aState, aDownload) {
-//@line 633 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/downloads.js"
+//@line 634 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/downloads.js"
   },
 
   onStateChange: function(aWebProgress, aRequest, aState, aStatus, aDownload) { },
