@@ -42,9 +42,9 @@ const PREF_GETADDONS_MAXRESULTS = "extensions.getAddons.maxResults";
 
 const kAddonPageSize = 5;
 
-//@line 48 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/extensions.js"
+//@line 48 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/extensions.js"
 const URI_GENERIC_ICON_XPINSTALL = "chrome://browser/skin/images/alert-addons-30.png";
-//@line 50 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/extensions.js"
+//@line 50 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/extensions.js"
 const ADDONS_NOTIFICATION_NAME = "addons";
 
 XPCOMUtils.defineLazyGetter(this, "AddonManager", function() {
@@ -87,6 +87,7 @@ var ExtensionsView = {
     item.setAttribute("name", aAddon.name);
     item.setAttribute("version", aAddon.version);
     item.setAttribute("iconURL", aAddon.iconURL);
+    item.setAttribute("class", "panel-listitem");
     return item;
   },
 
@@ -219,11 +220,6 @@ var ExtensionsView = {
     let os = Services.obs;
     os.addObserver(this, "addon-update-started", false);
     os.addObserver(this, "addon-update-ended", false);
-
-    if (!Services.prefs.getBoolPref("extensions.hideUpdateButton"))
-      document.getElementById("addons-update-all").hidden = false;
-
-//@line 235 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/extensions.js"
   },
 
   delayedInit: function ev__delayedInit() {
@@ -251,6 +247,9 @@ var ExtensionsView = {
     this._strings["addonType.locale"] = strings.GetStringFromName("addonType.8");
     this._strings["addonType.search"] = strings.GetStringFromName("addonType.1024");
 
+    if (!Services.prefs.getBoolPref("extensions.hideUpdateButton"))
+      document.getElementById("addons-update-all").hidden = false;
+
     let self = this;
     setTimeout(function() {
       self.getAddonsFromLocal();
@@ -264,6 +263,8 @@ var ExtensionsView = {
     os.removeObserver(this, "addon-update-ended");
 
     AddonManager.removeInstallListener(this._dloadmgr);
+
+    this.hideAlerts();
   },
 
   hideOnSelect: function ev_handleEvent(aEvent) {
@@ -298,7 +299,7 @@ var ExtensionsView = {
     listitem.setAttribute("appDisabled", aAddon.appDisabled);
     listitem.setAttribute("appManaged", appManaged);
     listitem.setAttribute("description", aAddon.description);
-    listitem.setAttribute("optionsURL", aAddon.optionsURL);
+    listitem.setAttribute("optionsURL", aAddon.optionsURL || "");
     listitem.setAttribute("opType", opType);
     listitem.setAttribute("updateable", updateable);
     listitem.setAttribute("isReadonly", !uninstallable);
@@ -316,7 +317,9 @@ var ExtensionsView = {
       let strings = Strings.browser;
       let anyUpdateable = false;
       for (let i = 0; i < items.length; i++) {
-        let listitem = self._createLocalAddon(items[i]);
+        let listitem = self.getElementForAddon(items[i].id)
+        if (!listitem)
+          listitem = self._createLocalAddon(items[i]);
         if ((items[i].permissions & AddonManager.PERM_CAN_UPGRADE) > 0)
           anyUpdateable = true;
 
@@ -420,6 +423,10 @@ var ExtensionsView = {
     } else if (aItem.getAttribute("type") == "theme") {
       aItem.addon.userDisabled = true;
       aItem.setAttribute("isDisabled", true);
+    } else if (aItem.getAttribute("type") == "locale") {
+      Services.prefs.clearUserPref("general.useragent.locale");
+      aItem.addon.userDisabled = true;
+      aItem.setAttribute("isDisabled", true);
     } else {
       aItem.addon.userDisabled = true;
       opType = this._getOpTypeForOperations(aItem.addon.pendingOperations);
@@ -538,7 +545,7 @@ var ExtensionsView = {
   appendSearchResults: function(aAddons, aShowRating, aShowCount) {
     let urlproperties = [ "iconURL", "homepageURL" ];
     let foundItem = false;
-    let appendedAddons = 0;
+    let appendedAddons = [];
     for (let i = 0; i < aAddons.length; i++) {
       let addon = aAddons[i];
 
@@ -556,7 +563,6 @@ var ExtensionsView = {
           continue;
       }
 
-      appendedAddons++;
       // Convert the numeric type to a string
       let types = {"2":"extension", "4":"theme", "8":"locale"};
       addon.type = types[addon.type];
@@ -571,7 +577,7 @@ var ExtensionsView = {
         listitem.setAttribute("rating", addon.averageRating);
 
       let item = this.addItem(listitem, "repo");
-
+      appendedAddons.push(listitem);
       // Hide any overflow add-ons. The user can see them later by pressing the
       // "See More" button
       aShowCount--;
@@ -613,6 +619,7 @@ var ExtensionsView = {
 
     let whatare = document.createElement("richlistitem");
     whatare.setAttribute("typeName", "banner");
+    whatare.setAttribute("class", "panel-listitem");
     whatare.setAttribute("label", strings.GetStringFromName("addonsWhatAre.label"));
 
     let desc = strings.GetStringFromName("addonsWhatAre.description");
@@ -648,7 +655,7 @@ var ExtensionsView = {
     // can see more by pressing the "Show More" button
     this.appendSearchResults(aRecommendedAddons, false, aRecommendedAddons.length);
     let minOverflow = (aRecommendedAddons.length >= kAddonPageSize ? 0 : kAddonPageSize);
-    let numAdded = this.appendSearchResults(aBrowseAddons, true, minOverflow);
+    let numAdded = this.appendSearchResults(aBrowseAddons, true, minOverflow).length;
 
     let totalAddons = aRecommendedAddons.length + numAdded;
 
@@ -680,10 +687,10 @@ var ExtensionsView = {
       return;
     }
 
-    let firstItem = this.appendSearchResults(aAddons, true);
-    if (aSelectFirstResult) {
-      this._list.selectItem(firstItem);
-      this._list.scrollBoxObject.scrollToElement(firstItem);
+    let firstAdded = this.appendSearchResults(aAddons, true)[0];
+    if (aSelectFirstResult && firstAdded) {
+      this._list.selectItem(firstAdded);
+      this._list.scrollBoxObject.scrollToElement(firstAdded);
     }
 
     let formatter = Cc["@mozilla.org/toolkit/URLFormatterService;1"].getService(Ci.nsIURLFormatter);
@@ -827,12 +834,19 @@ var ExtensionsView = {
         let alerts = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
         alerts.showAlertNotification(URI_GENERIC_ICON_XPINSTALL, strings.GetStringFromName("alertAddons"),
                                      aMessage, true, "", observer, ADDONS_NOTIFICATION_NAME);
+
+        // Use a preference to help us cleanup this notification in case we don't shutdown correctly
+        Services.prefs.setBoolPref("browser.notifications.pending.addons", true);
+        Services.prefs.savePrefFile(null);
       }
     }
   },
 
   hideAlerts: function ev_hideAlerts() {
-//@line 848 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/extensions.js"
+//@line 854 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/extensions.js"
+
+    // Keep our preference in sync
+    Services.prefs.clearUserPref("browser.notifications.pending.addons");
   },
 };
 
@@ -845,7 +859,7 @@ function searchFailed() {
 
   let failLabel = strings.formatStringFromName("addonsSearchFail.label",
                                              [brand.GetStringFromName("brandShortName")], 1);
-  let failButton = strings.GetStringFromName("addonsSearchFail.button");
+  let failButton = strings.GetStringFromName("addonsSearchFail.retryButton");
   ExtensionsView.displaySectionMessage("repo", failLabel, failButton, true);
 }
 
@@ -880,7 +894,7 @@ var AddonSearchResults = {
   selectFirstResult: false,
 
   searchSucceeded: function(aAddons, aAddonCount, aTotalResults) {
-    ExtensionsView.displaySearchResults(aAddons, aTotalResults, false, this.selectFirstResult);
+    ExtensionsView.displaySearchResults(aAddons, aTotalResults, this.selectFirstResult);
   },
 
   searchFailed: searchFailed
@@ -892,7 +906,6 @@ function AddonInstallListener() {
 }
 
 AddonInstallListener.prototype = {
-
   onInstallEnded: function(aInstall, aAddon) {
     let needsRestart = false;
     let mode = "";
@@ -904,11 +917,15 @@ AddonInstallListener.prototype = {
       mode = "normal";
     }
 
+    this._clearRecommendedCache();
+
     // if we already have a mode, then we need to show a restart notification
     // otherwise, we are likely a bootstrapped addon
     if (needsRestart)
       ExtensionsView.showRestart(mode);
-    this._showInstallCompleteAlert(true, needsRestart);
+
+    if (aAddon.type != "locale")
+      this._showInstallCompleteAlert(true, needsRestart);
 
     // only do this if the view has already been inited
     if (!ExtensionsView._list)
@@ -979,7 +996,8 @@ AddonInstallListener.prototype = {
   onDownloadProgress: function xpidm_onDownloadProgress(aInstall) {
     let element = ExtensionsView.getElementForAddon(aInstall.sourceURI.spec);
 
-//@line 999 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/chrome/content/extensions.js"
+//@line 1012 "/usr/ports/pobj/fennec-9.0/mozilla-release/mobile/chrome/content/extensions.js"
+
     if (!element)
       return;
 
@@ -1036,4 +1054,12 @@ AddonInstallListener.prototype = {
 
     ExtensionsView.showAlert(strings.GetStringFromName(stringName), !aNeedsRestart);
   },
+
+  _clearRecommendedCache: function xpidm_clearRecommendedCache() {
+    let dirService = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
+    let file = dirService.get("ProfD", Ci.nsILocalFile);
+    file.append("recommended-addons.json");
+    if (file.exists())
+      file.remove(false);
+  }
 };
