@@ -1,4 +1,4 @@
-//@line 40 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/toolkit/content/customizeToolbar.js"
+//@line 40 "/usr/obj/ports/fennec-9.0/mozilla-release/toolkit/content/customizeToolbar.js"
 
 const kRowMax = 4;
 
@@ -191,7 +191,7 @@ function wrapToolbarItems()
 {
   forEachCustomizableToolbar(function (toolbar) {
     Array.forEach(toolbar.childNodes, function (item) {
-//@line 236 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/toolkit/content/customizeToolbar.js"
+//@line 236 "/usr/obj/ports/fennec-9.0/mozilla-release/toolkit/content/customizeToolbar.js"
       if (isToolbarItem(item)) {
         let wrapper = wrapToolbarItem(item);
         cleanupItemForToolbar(item, wrapper);
@@ -698,15 +698,17 @@ function onToolbarDragExit(aEvent)
 
 function onToolbarDragStart(aEvent)
 {
-  var documentId = gToolboxDocument.documentElement.id;
-
   var item = aEvent.target;
-  while (item && item.localName != "toolbarpaletteitem")
+  while (item && item.localName != "toolbarpaletteitem") {
+    if (item.localName == "toolbar")
+      return;
     item = item.parentNode;
+  }
 
   item.setAttribute("dragactive", "true");
 
   var dt = aEvent.dataTransfer;
+  var documentId = gToolboxDocument.documentElement.id;
   dt.setData("text/toolbarwrapper-id/" + documentId, item.firstChild.id);
   dt.effectAllowed = "move";
 }
