@@ -35,7 +35,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-//@line 39 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 39 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // For browser.xml binding
 //
@@ -72,10 +72,12 @@ pref("toolkit.zoomManager.zoomValues", ".2,.3,.5,.67,.8,.9,1,1.1,1.2,1.33,1.5,1.
 
 // Device pixel to CSS px ratio, in percent. Set to -1 to calculate based on display density.
 pref("browser.viewport.scaleRatio", -1);
+pref("browser.viewport.desktopWidth", 980);
 
-/* use custom widget for html:select */
-pref("ui.use_native_popup_windows", true);
-
+//@line 79 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
+// On desktop builds, simulate an MDPI tablet by default.
+pref("layout.css.dpi", 160);
+//@line 86 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 /* allow scrollbars to float above chrome ui */
 pref("ui.scrollbarsCanOverlapContent", 1);
 
@@ -116,7 +118,7 @@ pref("network.http.max-connections", 6);
 pref("network.http.max-connections-per-server", 4);
 pref("network.http.max-persistent-connections-per-server", 4);
 pref("network.http.max-persistent-connections-per-proxy", 4);
-//@line 122 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 129 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // See bug 545869 for details on why these are set the way they are
 pref("network.buffer.cache.count", 24);
@@ -131,19 +133,19 @@ pref("browser.display.remotetabs.timeout", 10);
 /* session history */
 pref("browser.sessionhistory.max_total_viewers", 1);
 pref("browser.sessionhistory.max_entries", 50);
-pref("browser.sessionhistory.optimize_eviction", true);
 
 /* session store */
 pref("browser.sessionstore.resume_session_once", false);
 pref("browser.sessionstore.resume_from_crash", true);
 pref("browser.sessionstore.resume_from_crash_timeout", 60); // minutes
 pref("browser.sessionstore.interval", 10000); // milliseconds
-pref("browser.sessionstore.max_tabs_undo", 5);
+pref("browser.sessionstore.max_tabs_undo", 1);
 
 /* these should help performance */
 pref("mozilla.widget.force-24bpp", true);
 pref("mozilla.widget.use-buffer-pixmap", true);
 pref("mozilla.widget.disable-native-theme", true);
+pref("layout.reflow.synthMouseMove", false);
 
 /* download manager (don't show the window or alert) */
 pref("browser.download.useDownloadDir", true);
@@ -164,13 +166,17 @@ pref("alerts.slideIncrementTime", 10);
 pref("alerts.totalOpenTime", 6000);
 pref("alerts.height", 50);
 
+/* download helper */
+pref("browser.helperApps.deleteTempFileOnExit", false);
+
 /* password manager */
 pref("signon.rememberSignons", true);
 pref("signon.expireMasterPassword", false);
 pref("signon.SignonFileName", "signons.txt");
 
 /* form helper */
-pref("formhelper.enabled", true);
+// 0 = disabled, 1 = enabled, 2 = dynamic depending on screen size
+pref("formhelper.mode", 2);
 pref("formhelper.autozoom", true);
 pref("formhelper.autozoom.caret", true);
 pref("formhelper.restore", false);
@@ -181,10 +187,6 @@ pref("findhelper.autozoom", true);
 /* autocomplete */
 pref("browser.formfill.enable", true);
 
-/* microsummaries */
-pref("browser.microsummary.enabled", false);
-pref("browser.microsummary.updateGenerators", false);
-
 /* spellcheck */
 pref("layout.spellcheckDefault", 0);
 
@@ -215,6 +217,10 @@ pref("extensions.getAddons.search.url", "https://services.addons.mozilla.org/%LO
 pref("extensions.getAddons.browseAddons", "https://addons.mozilla.org/%LOCALE%/mobile/");
 pref("extensions.getAddons.get.url", "https://services.addons.mozilla.org/%LOCALE%/mobile/api/%API_VERSION%/search/guid:%IDS%?src=mobile&appOS=%OS%&appVersion=%VERSION%&tMain=%TIME_MAIN%&tFirstPaint=%TIME_FIRST_PAINT%&tSessionRestored=%TIME_SESSION_RESTORED%");
 
+/* preference for the locale picker */
+pref("extensions.getLocales.get.url", "");
+pref("extensions.compatability.locales.buildid", "0");
+
 /* blocklist preferences */
 pref("extensions.blocklist.enabled", true);
 pref("extensions.blocklist.interval", 86400);
@@ -362,14 +368,11 @@ pref("privacy.item.geolocation", true);
 pref("privacy.item.siteSettings", true);
 pref("privacy.item.syncAccount", true);
 
-//@line 370 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 381 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // URL to the Learn More link XXX this is the firefox one.  Bug 495578 fixes this.
 pref("browser.geolocation.warning.infoURL", "http://www.mozilla.com/%LOCALE%/firefox/geolocation/");
 
-// base url for the wifi geolocation network provider
-pref("geo.wifi.uri", "https://www.google.com/loc/json");
-
 // enable geo
 pref("geo.enabled", true);
 
@@ -380,9 +383,14 @@ pref("content.sink.pending_event_mode", 0);
 pref("content.sink.perf_deflect_count", 1000000);
 pref("content.sink.perf_parse_time", 50000000);
 
-pref("javascript.options.mem.gc_frequency", 300);
+// Disable methodjit in chrome to save memory
+pref("javascript.options.methodjit.chrome",  false);
+
 pref("javascript.options.mem.high_water_mark", 32);
-pref("javascript.options.methodjit.chrome", false);
+
+// Disable the JS engine's gc on memory pressure, since we do one in the mobile
+// browser (bug 669346).
+pref("javascript.options.gc_on_memory_pressure", false);
 
 pref("dom.max_chrome_script_run_time", 0); // disable slow script dialog for chrome
 pref("dom.max_script_run_time", 20);
@@ -390,6 +398,8 @@ pref("dom.max_script_run_time", 20);
 // JS error console
 pref("devtools.errorconsole.enabled", false);
 
+pref("browser.ui.layout.tablet", -1); // on: 1, off: 0, auto: -1
+
 // kinetic tweakables
 pref("browser.ui.kinetic.updateInterval", 16);
 pref("browser.ui.kinetic.exponentialC", 1400);
@@ -416,16 +426,16 @@ pref("browser.ui.touch.bottom", 4);
 pref("browser.ui.touch.weight.visited", 120); // percentage
 
 // plugins
-//@line 426 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 441 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("plugin.disable", true);
-//@line 428 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 443 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("dom.ipc.plugins.enabled", true);
 
 // process priority
 // higher values give content process less CPU time
-//@line 435 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 450 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("dom.ipc.content.nice", 1);
-//@line 437 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 452 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // product URLs
 // The breakpad report server to link to in about:crashes
@@ -436,15 +446,10 @@ pref("app.support.baseURL", "http://support.mozilla.com/mobile");
 pref("app.feedbackURL", "http://input.mozilla.com/feedback/");
 pref("app.privacyURL", "http://www.mozilla.com/%LOCALE%/m/privacy.html");
 pref("app.creditsURL", "http://www.mozilla.org/credits/");
-//@line 451 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 466 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("app.featuresURL", "http://www.mozilla.com/%LOCALE%/mobile/features/");
 pref("app.faqURL", "http://www.mozilla.com/%LOCALE%/mobile/faq/");
-//@line 454 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
-
-pref("app.promo.spark.baseURL", "http://spark.mozilla.org");
-//@line 457 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
-pref("app.promo.spark.endDate", "2011-05-01");
-//@line 461 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 469 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // Name of alternate about: page for certificate errors (when undefined, defaults to about:neterror)
 pref("security.alternate_certificate_error_page", "certerror");
@@ -481,16 +486,16 @@ pref("ui.window", "#efebe7");
 pref("ui.windowtext", "#101010");
 pref("ui.windowframe", "#efebe7");
 
-//@line 498 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 506 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("browser.search.param.yahoo-fr", "moz35");
 pref("browser.search.param.yahoo-fr-cjkt", "moz35");
 pref("browser.search.param.yahoo-fr-ja", "mozff");
-//@line 502 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 510 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 /* app update prefs */
 pref("app.update.timer", 60000); // milliseconds (1 min)
 
-//@line 507 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 515 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("app.update.enabled", true);
 pref("app.update.timerFirstInterval", 20000); // milliseconds
 pref("app.update.auto", false);
@@ -498,27 +503,24 @@ pref("app.update.channel", "default");
 pref("app.update.mode", 1);
 pref("app.update.silent", false);
 pref("app.update.url", "https://aus2.mozilla.org/update/4/%PRODUCT%/%VERSION%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/%PLATFORM_VERSION%/update.xml");
-pref("app.update.nagTimer.restart", 86400);
 pref("app.update.promptWaitTime", 43200);
 pref("app.update.idletime", 60);
 pref("app.update.showInstalledUI", false);
 pref("app.update.incompatible.mode", 0);
 pref("app.update.download.backgroundInterval", 0);
 
-//@line 522 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 529 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("app.update.interval", 86400);
 pref("app.update.url.manual", "http://www.mozilla.com/%LOCALE%/m/");
 pref("app.update.url.details", "http://www.mozilla.com/%LOCALE%/mobile/releases/");
-//@line 531 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 538 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // replace newlines with spaces on paste into single-line text boxes
 pref("editor.singleLine.pasteNewlines", 2);
 
-//@line 544 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
-
-//@line 546 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
-pref("browser.sync.enabled", true);
+//@line 551 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
+//@line 553 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 // sync service
 pref("services.sync.client.type", "mobile");
 pref("services.sync.registerEngines", "Tab,Bookmarks,Form,History,Password,Prefs");
@@ -535,23 +537,25 @@ pref("services.sync.prefs.sync.lightweightThemes.isThemeSelected", true);
 pref("services.sync.prefs.sync.lightweightThemes.usedThemes", true);
 pref("services.sync.prefs.sync.network.cookie.cookieBehavior", true);
 pref("services.sync.prefs.sync.permissions.default.image", true);
+pref("services.sync.prefs.sync.privacy.donottrackheader.enabled", true);
 pref("services.sync.prefs.sync.signon.rememberSignons", true);
-//@line 566 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 572 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 // threshold where a tap becomes a drag, in 1/240" reference pixels
 // The names of the preferences are to be in sync with nsEventStateManager.cpp
 pref("ui.dragThresholdX", 25);
 pref("ui.dragThresholdY", 25);
 
-//@line 575 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 583 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 pref("layers.acceleration.disabled", true);
-//@line 577 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/mobile/app/mobile.js"
+//@line 585 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
 
 pref("notification.feature.enabled", true);
 
 // prevent tooltips from showing up
 pref("browser.chrome.toolbar_tips", false);
-pref("indexedDB.feature.enabled", false);
+pref("indexedDB.feature.enabled", true);
+pref("dom.indexedDB.warningQuota", 5);
 
 // prevent video elements from preloading too much data
 pref("media.preload.default", 1); // default to preload none
@@ -566,5 +570,29 @@ pref("widget.ime.android.fullscreen_threshold", 250); // in hundreths of inches
 // optimize images memory usage
 pref("image.mem.decodeondraw", true);
 pref("content.image.allow_locking", false);
-pref("image.mem.min_discard_timeout_ms", 20000);
+pref("image.mem.min_discard_timeout_ms", 10000);
 
+// enable touch events interfaces
+pref("dom.w3c_touch_events.enabled", true);
+pref("dom.w3c_touch_events.safetyX", 0); // escape borders in units of 1/240"
+pref("dom.w3c_touch_events.safetyY", 120); // escape borders in units of 1/240"
+
+//@line 663 "/usr/obj/ports/fennec-9.0/mozilla-release/mobile/app/mobile.js"
+
+// True if this is the first time we are showing about:firstrun
+pref("browser.firstrun.show.uidiscovery", true);
+pref("browser.firstrun.show.localepicker", false);
+
+// True if you always want dump() to work
+//
+// On Android, you also need to do the following for the output
+// to show up in logcat:
+//
+// $ adb shell stop
+// $ adb shell setprop log.redirect-stdio true
+// $ adb shell start
+pref("browser.dom.window.dump.enabled", false);
+
+// controls if we want camera support
+pref("device.camera.enabled", true);
+pref("media.realtime_decoder.enabled", true);
