@@ -1,4 +1,4 @@
-//@line 45 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/downloads/content/downloads.js"
+//@line 45 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/downloads/content/downloads.js"
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Globals
@@ -68,8 +68,6 @@ let gStr = {
   stateBlockedParentalControls: "stateBlocked",
   stateBlockedPolicy: "stateBlockedPolicy",
   stateDirty: "stateDirty",
-  yesterday: "yesterday",
-  monthDate: "monthDate",
   downloadsTitleFiles: "downloadsTitleFiles",
   downloadsTitlePercent: "downloadsTitlePercent",
   fileExecutableSecurityWarningTitle: "fileExecutableSecurityWarningTitle",
@@ -251,7 +249,7 @@ function openDownload(aDownload)
       dontAsk = !pref.getBoolPref(PREF_BDM_ALERTONEXEOPEN);
     } catch (e) { }
 
-//@line 311 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/downloads/content/downloads.js"
+//@line 307 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/downloads/content/downloads.js"
 
     if (!dontAsk) {
       var strings = document.getElementById("downloadStrings");
@@ -435,7 +433,7 @@ function Startup()
     }
   }, false);
 
-//@line 501 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/downloads/content/downloads.js"
+//@line 495 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/downloads/content/downloads.js"
 }
 
 function Shutdown()
@@ -491,11 +489,11 @@ let gDownloadObserver = {
         }
         break;
       case "browser-lastwindow-close-granted":
-//@line 557 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/downloads/content/downloads.js"
+//@line 551 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/downloads/content/downloads.js"
         if (gDownloadManager.activeDownloadCount == 0) {
           setTimeout(gCloseDownloadManager, 0);
         }
-//@line 561 "/usr/obj/ports/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/downloads/content/downloads.js"
+//@line 555 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/downloads/content/downloads.js"
         break;
     }
   }
@@ -994,51 +992,10 @@ function updateTime(aItem)
   if (aItem.inProgress)
     return;
 
-  let dts = Cc["@mozilla.org/intl/scriptabledateformat;1"].
-            getService(Ci.nsIScriptableDateFormat);
-
-  // Figure out when today begins
-  let now = new Date();
-  let today = new Date(now.getFullYear(), now.getMonth(), now.getDate());
-
-  // Get the end time to display
   let end = new Date(parseInt(aItem.getAttribute("endTime")));
-
-  // Figure out if the end time is from today, yesterday, this week, etc.
-  let dateTime;
-  if (end >= today) {
-    // Download finished after today started, show the time
-    dateTime = dts.FormatTime("", dts.timeFormatNoSeconds,
-                              end.getHours(), end.getMinutes(), 0);
-  } else if (today - end < (24 * 60 * 60 * 1000)) {
-    // Download finished after yesterday started, show yesterday
-    dateTime = gStr.yesterday;
-  } else if (today - end < (6 * 24 * 60 * 60 * 1000)) {
-    // Download finished after last week started, show day of week
-    dateTime = end.toLocaleFormat("%A");
-  } else {
-    // Download must have been from some time ago.. show month/day
-    let month = end.toLocaleFormat("%B");
-    // Remove leading 0 by converting the date string to a number
-    let date = Number(end.toLocaleFormat("%d"));
-    dateTime = replaceInsert(gStr.monthDate, 1, month);
-    dateTime = replaceInsert(dateTime, 2, date);
-  }
-
-  aItem.setAttribute("dateTime", dateTime);
-
-  // Set the tooltip to be the full date and time
-  let dateTimeTip = dts.FormatDateTime("",
-                                       dts.dateFormatLong,
-                                       dts.timeFormatNoSeconds,
-                                       end.getFullYear(),
-                                       end.getMonth() + 1,
-                                       end.getDate(),
-                                       end.getHours(),
-                                       end.getMinutes(),
-                                       0); 
-
-  aItem.setAttribute("dateTimeTip", dateTimeTip);
+  let [dateCompact, dateComplete] = DownloadUtils.getReadableDates(end);
+  aItem.setAttribute("dateTime", dateCompact);
+  aItem.setAttribute("dateTimeTip", dateComplete);
 }
 
 /**
@@ -1144,7 +1101,6 @@ function buildDownloadList(aForceBuild)
   gBuilder = setTimeout(function() {
     // Start building the list and select the first item
     stepListBuilder(1);
-    gDownloadsView.selectedIndex = 0;
 
     // We just tried to add a single item, so we probably need to enable
     updateClearListButton();
