@@ -1,4 +1,6 @@
-//@line 37 "/usr/obj/i386/fennec-5.0/mozilla-mobile-5.0/toolkit/mozapps/extensions/content/eula.js"
+//@line 37 "/usr/ports/pobj/fennec-9.0/mozilla-release/toolkit/mozapps/extensions/content/eula.js"
+
+"use strict";
 
 function Startup() {
   var bundle = document.getElementById("extensionsStrings");
